=begin

earth_cut.rb

Local mail library for a mail forwarding control script
Copyright (C) 2001-2003 Masaharu FUJITA

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You may contact the author by:
        e-mail: m@fjts.org
        url: http://fjts.org/~m/Soft/Ruby/earth.rb/
=end

Byte_of_multi = 2
Byte_of_ascii = 1
OptimizedNumber = 7

# for OneWord
class EarthOneWord
  def initialize(string,max_byte = -1)
    @string = string
    @max_byte = max_byte

    @type = 'unknown'
    @one_word = ''
    @rest_string = nil
    @byte = 0
    raw_byte = 0
    string_byte = string.length

    @string.each_char do |char|
      # Multi character case
      if char.mbchar?
	if @one_word != '' # find OneWord
	  break
	end

	@type = 'multi'
	@one_word = char
	@byte = Byte_of_multi
	raw_byte += Byte_of_multi
	break
      end

      # Ascii character case
      raw_byte += Byte_of_ascii
      @type = 'ascii'

      # Space and tab and line feed case
      if /\s/ =~ char
	if @one_word != '' # find OneWord
	  break
	else
	  next
	end
      end

      # Another ascii character case
      @one_word << char
      @byte += Byte_of_ascii
    end # @string.each_char do |char|
    
    if string_byte >= raw_byte
      @rest_string = string[raw_byte .. -1]
    end
    check_special_type

    if @max_byte != -1 and @max_byte < @byte
      @type = 'ascii'
      @one_word = '<too big word>'
      @byte = @one_word.length
    end
  end # def initialize(string)

  # # Whether special type.
  def check_special_type
    # Whether phone number.
    flag = false

    if /\A[:(0-9]/ =~ @one_word
      flag = true
    end

    if flag
      if /[:(]?[0-9]{2,4}[-(]?[0-9]{2,4}[-)]?[0-9]{4}/ =~ @one_word
        if /\A:/ =~ @one_word
          # Case ":XXXXXXXXXXX"
          @one_word = @one_word.delete(":")
	  @byte -= 1
        end

	@type = 'spacial'
        @one_word = "TEL:" + @one_word
	@byte += "TEL:".length

	return
      end
    end

    # Whether URL.
    if /\Ahttp/ =~ @one_word
      # URL
      @type = 'special'
      return
    end

    # Whether E-mail address.
    if /.*@.*/ =~ @one_word
      @type = 'spacial'
      if /\Amailto:/ =~ @one_word
        @one_word = @one_word.sub(/mailto:/,'')
	@byte -= "mailto:".length
      end

      return
    end # if /.*@.*/ =~ @one_word
  end # def check_special_type

  def string
    @one_word
  end

  def type
    @type
  end

  def byte
    @byte
  end

  def rest
    @rest_string
  end
end # class EarthOneWord

# for cut string
class EarthCutString
  def initialize(type,string,cut_byte = -1,linefeedcharacter = "\n")
    if not string
      return nil
    end

    @type = type
    @string = string
    @linefeedcharacter = linefeedcharacter
    @cut_byte = cut_byte

    @return_string = ''
    @rest_string = nil
    @byte = 0

    if @type == 'J-PHONE'
      cut_for_jphone
    elsif @type == 'J-PHONE_CUTONLY'
      cut_for_jphone_cutonly
    else
      cut_for_normal
    end
  end # def initialize(type)

  # Cut for J-PHONE
  def cut_for_jphone
    # Control byte of ascii character to multi character of J-PHONE.
    ctrl_byte_of_jphone = 6

    before_string = nil
    euc_return_string = ''

    is_convert_to_multi = false
    is_reply_char = false
    is_redo = false
    is_max_byte = false
    type_end_of_return_string = 'ascii'
    jis_string_byte = 0
    ctrl_byte = 0

    tmp_string = ''
    tmp_string_byte = 0

    while @string != ''
      before_string = @string
      add_string = ''
      add_ctrl_byte = 0

      ow = EarthOneWord.new(@string,@cut_byte)
      @string = ow.rest
      if ow.string == '' or
	  euc_return_string == '' and ow.string == @linefeedcharacter
	next
      end

      if ow.type == 'multi' # Multi character case
	if tmp_string != ''
	  if is_convert_to_multi
	    # Convert to multibyte character
	    @string = ascii2multi(tmp_string) + before_string
	    tmp_string = ''
	    tmp_string_byte = 0
	    next
	  else
	    # Add a OneWord of ascii character 
	    add_string = tmp_string
	    type_end_of_return_string = 'ascii'
	    tmp_string = ''
	    tmp_string_byte = 0
	    @string = before_string # redo
	  end # if is_convert_to_multi
	else # Normal case
	  if type_end_of_return_string == 'ascii'
	    add_ctrl_byte += ctrl_byte_of_jphone
	  end

	  add_string = ow.string
	  type_end_of_return_string = 'multi'
	end # if tmp_string != ''
      else # Ascii character case
	if type_end_of_return_string == 'ascii'
          if euc_return_string == ''
	    add_string = ow.string
          else
	    add_string = ' ' + ow.string
          end
	  type_end_of_return_string = 'ascii'
        else # type_end_of_return_string == 'multi'
	  if tmp_string == ''
	    tmp_byte = ow.byte
	  else
	    tmp_byte = tmp_string_byte + 1 + ow.byte
	  end
	  
	  if tmp_byte < OptimizedNumber and ow.type != 'special'
	    if '' == tmp_string or /[:|>]+/ =~ tmp_string
	      tmp_string += ow.string
	      tmp_string_byte += ow.byte
	    else
	      tmp_string += ' ' + ow.string
	      tmp_string_byte += ow.byte + 1
	    end
	    is_convert_to_multi = true
	    next
	  else # Add about ascii character.
	    if '' == tmp_string
	      add_string = ow.string
	    elsif /[:|>]+/ =~ tmp_string
	      add_string = tmp_string + ow.string
	    else
	      add_string = tmp_string + ' ' + ow.string
	    end
	    type_end_of_return_string = 'ascii'
	    tmp_string = ''
	    tmp_string_byte = 0
	    is_convert_to_multi = false
	  end # if tmp_byte < OptimizedNumber and ow.type != 'special'
        end # if type_end_of_return_string == 'ascii'
      end # if ow.type == 'multi'

      # Check
      jis_string_byte = Kconv.tojis(euc_return_string + add_string).length
      if jis_string_byte + ctrl_byte + add_ctrl_byte > @cut_byte and
	  @cut_byte != -1
	is_max_byte = true
	break
      end
      
      euc_return_string += add_string
      ctrl_byte += add_ctrl_byte
    end # while @string

    @return_string = euc_return_string
    @byte = Kconv.tojis(euc_return_string).length + ctrl_byte
    if @string == '' and not is_max_byte
      @rest_string = nil
    else
      @rest_string = before_string
    end
  end # def cut_for_jphone

  # Cut for J-PHONE (cut only type)
  def cut_for_jphone_cutonly
    # Control byte of ascii character to multi character of J-PHONE.
    ctrl_byte_of_jphone = 6

    euc_return_string = ''
    type_end_of_return_string = 'ascii'
    jis_string_byte = 0
    ctrl_byte = 0
    linefeed_byte = 0
    raw_byte = 0
    is_max_byte = false

    @string.each_char do |add_string|
      add_ctrl_byte = 0

      # check ascii charactor or multibyte character.
      if add_string.mbchar?
	type_end_of_return_string = 'multi'
	add_raw_byte = 2

	if type_end_of_return_string == 'ascii'
	  add_ctrl_byte = ctrl_byte_of_jphone
	end
      else
	type_end_of_return_string = 'ascii'
	add_raw_byte = 1
      end # if add_string.mbchar?

      # for convert from \n to \r\n.
      if "\n" == add_string
	linefeed_byte += 1
      end

      # Check
      jis_string_byte =	Kconv.tojis(euc_return_string + add_string).length
      if jis_string_byte + ctrl_byte + linefeed_byte > @cut_byte and
	  @cut_byte != -1
	is_max_byte = true
	break
      end
      
      euc_return_string += add_string
      ctrl_byte += add_ctrl_byte
      raw_byte += add_raw_byte
    end # @string

    @return_string = euc_return_string
    @byte = Kconv.tojis(euc_return_string).length + ctrl_byte + linefeed_byte
    if @string.length <= raw_byte and not is_max_byte
      @rest_string = nil
    else
      @rest_string = @string[raw_byte .. -1]
    end
  end # def cut_for_jphone_raw

  # Cut for normal
  def cut_for_normal
    before_string = nil
    euc_return_string = ''

    is_redo = false
    type_end_of_return_string = 'ascii'
    jis_string_byte = 0

    tmp_string = ''
    tmp_string_byte = 0

    while @string != ''
      before_string = @string
      add_string = ''

      ow = EarthOneWord.new(@string,@cut_byte)
      @string = ow.rest
      if ow.string == '' or
	  euc_return_string == '' and ow.string == @linefeedcharacter
	next
      end

      if ow.type == 'multi' # Multi character case
	if tmp_string != ''
	  # Add a OneWord of ascii character 
	  add_string = tmp_string
	  type_end_of_return_string = 'ascii'
	  tmp_string = ''
	  tmp_string_byte = 0
	  @string = before_string # redo
	else # Normal case
	  add_string = ow.string
	  type_end_of_return_string = 'multi'
	end # if tmp_string != ''
      else # Ascii character case
	if type_end_of_return_string == 'ascii'
          if euc_return_string == ''
	    add_string = ow.string
          else
	    add_string = ' ' + ow.string
          end
	  type_end_of_return_string = 'ascii'
        else # type_end_of_return_string == 'multi'
	  if tmp_string == ''
	    tmp_byte = ow.byte
	  else
	    tmp_byte = tmp_string_byte + 1 + ow.byte
	  end
	  
	  if tmp_string == ''
	    add_string = ow.string
	  else
	    add_string = tmp_string + ' ' + ow.string
	  end
	  type_end_of_return_string = 'ascii'
	  tmp_string = ''
	  tmp_string_byte = 0
        end # if type_end_of_return_string == 'ascii'
      end # if ow.type == 'multi'

      # Check
      jis_string_byte = Kconv.tojis(euc_return_string + add_string).length
      if jis_string_byte > @cut_byte and @cut_byte != -1
	break
      end
      
      euc_return_string += add_string
    end # while @string

    @return_string = euc_return_string
    @byte = Kconv.tojis(euc_return_string).length
    if @string == ''
      @rest_string = nil
    else
      @rest_string = before_string
    end
  end # def cut_for_normal

  def string
    @return_string
  end

  def byte
    @byte
  end

  def rest
    @rest_string
  end
end # class EarthCutString
