=begin

earth_log.rb

Local mail library for a mail forwarding control script
Copyright (C) 2001-2003 Masaharu FUJITA

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You may contact the author by:
        e-mail: m@fjts.org
        url: http://fjts.org/~m/Soft/Ruby/earth.rb/
=end

# for logging.
class EarthLog
  def initialize(log_file, spool_max)
    # Getting current time.
    @time = Time.now

    @log_file = log_file
    @spool_max = spool_max
    @is_end = false
  end # def initialize

  def write_first
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "[" + @time.to_s + "]\n"
    ensure
      file.close
    end # begin
  end # def write_first

  def write_end
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "================================================================================"
      file.print "\n"
    ensure
      file.close
    end # begin

    # Backup of file.
    if FileTest::exist?(@log_file)
      if File.size(@log_file) > 300 * @spool_max
        File.rename(@log_file,@log_file + '.bak')
      end
    end
  end # def write_end

  def write_info(number,from,subject,tocc,date,match)
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "  Spool_No.: " + number.to_s + "\n"

      if from != nil
        file.print "  From: " + from + "\n"
      end

      if subject != nil
        file.print "  Subject: " + subject + "\n"
      end

      if tocc != nil
        file.print "  To: " + tocc + "\n"
      end

      if date != nil
        file.print "  Date: " + date + "\n"
      end

      if match != nil
        file.print "  Match: " + match + "\n"
      end
    ensure
      file.close
    end # begin
  end # def write_info

  def write_info_mail(number)
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "  Stats for No." + number.to_s
      file.print ": The infomaion of this mail was send. \n"
    ensure
      file.close
    end # begin
  end # def write_info_mail

  def write_no_send(number)
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "  Stats for No." + number.to_s
      file.print ": This mail did not send it.\n"
    ensure
      file.close
    end # begin
  end # def write_no_send

  def write_info_mail_in_suspend(number)
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "  Stats for No." + number.to_s
      file.print ": The infomaion of this mail was created.\n"
    ensure
      file.close
    end # begin
  end # def write_info_mail_in_suspend

  def write_normal(mail_number,number,just = false)
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "  Stats for No." + mail_number.to_s
      if just
	file.print ": This mail was just send.\n"
      else
	file.print ": This mail was divided into " + number.to_s
	file.print ", and they were sent.\n"
      end
    ensure
      file.close
    end # begin
  end # def write_normal

  def write_reget(number)
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "--------------------------------------------------------------------------------\n"
      file.print "CtrlCmd: Exec reget for " + number.to_s + ".\n"
    ensure
      file.close
    end # begin
  end # def write_reget

  def write_reply(number)
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "  Stats: Exec reply for " + number.to_s + ".\n"
    ensure
      file.close
    end # begin
  end # def write_reply

  def write_send(to)
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "  Stats: Exec send to " + to.join(',') + ".\n"
    ensure
      file.close
    end # begin
  end # def write_send

  def write_error(string)
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "  Error: " + string + "\n"
    ensure
      file.close
    end # begin
  end # def write_error

  def write_restart
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "--------------------------------------------------------------------------------\n"
      file.print "Restart mail forwarding.\n"
    ensure
      file.close
    end # begin
  end # def write_restart

  def write_normal_in_restart(mq_number,number)
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "  Mqueue stats for No." + mq_number.to_s
      file.print ": This mail was divided into " + number.to_s
      file.print ", and they were sent.\n"
    ensure
      file.close
    end # begin
  end # def write_normal_in_restart

  def write_suspend(mail_number,number)
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "  Stats for No." + mail_number.to_s
      file.print ": This mail was divided into " + number.to_s + ", "
      file.print "and they were saved to Mqueue directory.\n"
    ensure
      file.close
    end # begin
  end # def write_suspend

  def write_pop_start(pop_type,server,user,port)
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "--------------------------------------------------------------------------------\n"
      file.print "Start of " + pop_type
      file.print "(" + port.to_s + ")"
      file.print " access to " + server 
      file.print " as " + user
      file.print " at " + @time.to_s + ".\n"
      file.print "================================================================================"
    ensure
      file.close
    end # begin
  end # def write_pop_start

  def write_pop_end(pop_type,server,user,port)
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "End of " + pop_type
      file.print "(" + port.to_s + ")"
      file.print " access to " + server 
      file.print " as " + user + ".\n"
      file.print "--------------------------------------------------------------------------------\n"
    ensure
      file.close
    end # begin
  end # def write_pop_end

  def write_pop_error(pop_type,server,user,port)
    begin
      file = open(@log_file, "a")
    rescue
    else
      file.print "--------------------------------------------------------------------------------\n"
      file.print "Error of " + pop_type
      file.print "(" + port.to_s + ")"
      file.print " access to " + server 
      file.print " as " + user
      file.print " at " + @time.to_s + ".\n"
      file.print "--------------------------------------------------------------------------------\n"
    ensure
      file.close
    end # begin
  end # def write_pop_error

  def set_end
    @is_end = true
  end

  def init
    @is_end = false
  end

  def time
    @time
  end

  def is_end
    @is_end
  end
end # class EarthLog

# for print with debug
class EarthPrint
  def initialize(header,subject=nil,string=nil,byte=0,count=nil)
    @header = header
    @subject = subject
    @string = string
    @byte = byte
    @count = count
  end # def initialize

  def header_tag
    print "*** Header part ****************************************************************\n"
  end

  def header
    @header.each do |line|
      print line
    end # @header.each do |line|
    print "\n"
  end # def header

  def body_tag
    print "**** Body part *****************************************************************\n"
  end

  def body
    if @count
      print "== No."
      print @count
      print " ==================================================== "
    else
      print "== Infomation ============================================== "
    end

    print @byte
    print " byte ==========\n"
    if @subject != nil
      print "Subject: " + @subject + "\n"
    end

    print "--------------------------------------------------------------------------------\n"
    print @string
    print "\n--------------------------------------------------------------------------------\n\n"
  end # def body
end # class EarthPrint
